#include "print.h"

void printErrMessage (char *message, char *detail) {
  fprintf(stderr, "%s: %s\n", message, detail);
}


void printArray (char *label, int *array, int start, int end, int m1, int m2) {
  int i;

  printf("%s: [", label);
  for (i = start; i <= end; ++i) {
    if (i == m1 || i == m2) printf("*");
    printf("%d", array[i]);
    if (i != end) printf(", ");
  }
  printf("]\n");
}


void printBinaryTreeSub(int depth, char *str, BinaryTreeNode *node, int islast, 
			BinaryTreeNode *marker1, BinaryTreeNode *marker2) {
  if (node == NULL) {
    if (!islast) printf("%s \n", str);
    return;
  } //leaf

  int i;

  printf("%s", str);
  if (!islast) printf("%c%d\n",
		     (node == marker1 || node == marker2) ? '*':' ',
		     node->val);
  else printf("%c%d\n",
	      (node == marker1 || node == marker2) ? '*':' ',
	      node->val);

  int len = strlen(str) + 6; //multibyte char?

  char *strr = malloc(len * sizeof(char));
  char *strl = malloc(len * sizeof(char));

  strcpy(strr, str);
  strcpy(strl, str);
  strcpy(&strr[strlen(str)], islast ? "    " : "   ");
  strcpy(&strl[strlen(str)], islast ? "    " : "   ");
  
  printBinaryTreeSub(depth + 1, strr, node->right, (node->left == NULL), marker1, marker2);
  printBinaryTreeSub(depth + 1, strl, node->left, 1, marker1, marker2);

  free(strr);
  free(strl);
}

void printBinaryTree (char *label, BinaryTreeNode *node,
		      BinaryTreeNode *marker1, BinaryTreeNode *marker2) {
  printf("[%s]\n", label);
  if (node == NULL) return;
  printf("%d\n", node->val);
  printBinaryTreeSub(0, "", node->right, (node->left == NULL), marker1, marker2);
  printBinaryTreeSub(0, "", node->left, 1, marker2, marker2);
}

void printMFSetsSub (int depth, char *str, MFSetNode *node, int islast,
		     MFSetNode *marker1, MFSetNode *marker2) {
  if (node == NULL) return; //leaf

  int i;
  printf("%s", str);
  if (!islast) printf("%c%d\n",
		      (node == marker1 || node == marker2) ? '*':' ',
		      node->val);
  else printf("%c%d\n",
	      (node == marker1 || node == marker2) ? '*':' ',
	      node->val);

  int len = strlen(str) + 6; //multibyte char?
  for (i = 0; node->children[i] != NULL; ++i);
  int childsize = i;

  char **strs = malloc(childsize * len * sizeof(char));
  for (i = 0; i < childsize; ++i) {
    strs[i] = malloc(len * sizeof(char));
    strcpy(strs[i], str);
    strcpy(&strs[i][strlen(str)], islast ? "    " : "   ");
  }
  
  int l;
  for (i = 0; i < childsize; ++i) {
    l = (node->children[i+1] == NULL);
    printMFSetsSub(depth + 1, strs[i], node->children[i], l,
		   marker1, marker2);
  }

  for (i = 0; i < childsize; ++i)
    free(strs[i]);

  free(strs);
}


void printMFSets (char *label, 
		  MFSet **sets, MFSetNode **val_to_node, int size,
		  MFSetNode *marker1, MFSetNode *marker2) {

  // Setting children
  int i, j; MFSetNode *node;
  for (i = 0; i < size; ++i) {
    if (val_to_node[i] != NULL) {
      for (j = 0; j <= size; ++j) (val_to_node[i]->children)[j] = NULL;
    }
  }

  for (i = 0; i < size; ++i) {
    node = val_to_node[i];
    if (node != NULL && node->parent != NULL) {
      for (j = 0; (node->parent->children)[j] != NULL; ++j);
      (node->parent->children)[j] = node;
    }
  }

  printf("%s:\n", label);
  for (i = 0; i < 256; ++i) {
    if (sets[i] == NULL) continue;
    printf("%c\n", (char)i);
    printMFSetsSub(0, "", sets[i]->root, 1, marker1, marker2);
    printf("\n");
  }

}


// GraphVis̉@\

/*
void printBinaryTreeSub1 (FILE *stream,
			  BinaryTreeNode *node, 
			  BinaryTreeNode *marker1, BinaryTreeNode *marker2) {

  if (node == NULL) return;
  else if (node == marker1 || node == marker2)
    fprintf(stream, "  \"%p\" [peripheries = 2, label = %d];\n", node, node->val);
  else
    fprintf(stream, "  \"%p\" [label = %d];\n", node, node->val);

  printBinaryTreeSub1 (stream, node->left, marker1, marker2);
  printBinaryTreeSub1 (stream, node->right, marker1, marker2);
}


void printBinaryTreeSub2 (FILE *stream, BinaryTreeNode *node) {
  if (node == NULL) return;

  if (node->left != NULL) {
    fprintf(stream, "  \"%p\" -> \"%p\";\n", node, node->left);
    printBinaryTreeSub2 (stream, node->left);
  }

  if (node->right != NULL) {
    fprintf(stream, "  \"%p\" -> \"%p\";\n", node, node->right);
    printBinaryTreeSub2 (stream, node->right);
  }
}


void printBinaryTree (char *filename,
		      char *label, BinaryTreeNode *root, 
		      BinaryTreeNode *marker1, BinaryTreeNode *marker2) {
  FILE *stream;
  if ((stream = fopen(filename, "w")) == NULL) {
    printErrMessage("IO Error", "Cannot open the file.");
    return;
  }
  fprintf(stream, "digraph BinaryTree {\n");
  fprintf(stream, "  graph[label = \"%s\", labelloc = t];\n\n", label);

  printBinaryTreeSub1 (stream, root, marker1, marker2);

  fprintf(stream, "  \n");
  printBinaryTreeSub2 (stream, root);

  fprintf(stream, "}\n");

  fclose(stream);
}


void printMFSetsSub(FILE *stream, char setname, int marked,
		    MFSet *set, MFSetNode **val_to_node, int size) {
  fprintf(stream, "  subgraph cluster_%c {\n", setname);

  int i; MFSetNode *node, *tmpnode;
  fprintf(stream, "    \"%c\" [shape = box", setname);
  if (marked)
    fprintf(stream, ", peripheries = 2");
  fprintf(stream, "];\n");

  for (i = 0; i < size; ++i) {
    node = val_to_node[i];

    if (node == NULL) continue;
    tmpnode = node;
    while(tmpnode->parent != NULL){
      tmpnode = tmpnode->parent;
    }

    if (tmpnode->set == set) {
      if (node->parent != NULL)
	fprintf(stream, "    \"%d\" -> \"%d\";\n", node->val, node->parent->val);
      else
	fprintf(stream, "    \"%d\" -> \"%c\";\n", node->val, setname);
    }
  }
  
  fprintf(stream, "  }\n\n", setname);
}


void printMFSets (char *filename, char *label, 
		  MFSet **sets, MFSetNode **val_to_node, int size,
		  MFSet *marker1, MFSet *marker2) {
  FILE *stream;

  if ((stream = fopen(filename, "w")) == NULL) {
    printErrMessage("IO Error", "Cannot open the file.");
    return;
  }

  fprintf(stream, "digraph MergeFindSets {\n");
  fprintf(stream, "  graph[label = \"%s\", labelloc = t, rankdir = BT];\n\n", label);

  int i;
  for (i = 0; i < 256; ++i) {
    if (sets[i] == NULL) continue;
    printMFSetsSub(stream, (char)i, (sets[i] == marker1 || sets[i] == marker2),
		   sets[i], val_to_node, size);
  }
  fprintf(stream, "}\n");

  fclose(stream);
}

*/
